# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2024-2025, by Samuel Williams.

source "https://rubygems.org"

gem "openssl"
gem "async"
gem "io-endpoint"
gem "io-stream", path: "../../"

gem "localhost"
